from django.apps import AppConfig


class CrudViewsConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'crud_views'

    def ready(self):
        import crud_views.checks  # noqa



from crud_views.lib.view import CrudViewPermissionRequiredMixin
from .utils import PolymorphicCrudViewMixin
from ..views import DetailView


class PolymorphicDetailView(PolymorphicCrudViewMixin, DetailView):
    pass


class PolymorphicDetailViewPermissionRequired(CrudViewPermissionRequiredMixin, PolymorphicDetailView):  # this file
    cv_permission = "view"



from typing import Iterable

from django.views import generic
from django_object_detail.views import ObjectDetailMixin

from crud_views.lib.check import Check
from crud_views.lib.settings import crud_views_settings
from crud_views.lib.view import CrudView, CrudViewPermissionRequiredMixin


class DetailView(ObjectDetailMixin, CrudView, generic.DetailView):
    template_name = "crud_views/view_detail.html"

    cv_key = "detail"
    cv_path = "detail"
    cv_context_actions = crud_views_settings.detail_context_actions

    # texts and labels
    cv_header_template: str | None = "crud_views/snippets/header/detail.html"
    cv_paragraph_template: str | None = "crud_views/snippets/paragraph/detail.html"
    cv_action_label_template: str | None = "crud_views/snippets/action/detail.html"
    cv_action_short_label_template: str | None = "crud_views/snippets/action_short/detail.html"

    # icons
    cv_icon_action = "fa-regular fa-eye"

    @classmethod
    def checks(cls) -> Iterable[Check]:
        """
        Iterator of checks for the view
        """
        yield from super().checks()


class DetailViewPermissionRequired(CrudViewPermissionRequiredMixin, DetailView):  # this file
    cv_permission = "view"



from functools import cached_property
from typing import List, Dict, Generator, Type

from django.contrib.contenttypes.models import ContentType
from django.db.models import Model
from django.forms import ModelForm
from polymorphic.models import PolymorphicModel


def get_polymorphic_child_models(model: PolymorphicModel) -> List[Type[PolymorphicModel]]:
    """
    Get all child models of a polymorphic model.
    """
    assert issubclass(model, PolymorphicModel), "not a polymorphic model"
    def subclasses(m: Type[PolymorphicModel]) -> Generator[Type[PolymorphicModel], None, None]:
        for sm in m.__subclasses__():
            yield sm
            yield from subclasses(sm)

    child_models = list(subclasses(model))
    return child_models


def get_polymorphic_child_models_content_types(model: PolymorphicModel) -> Dict[Type[Model], ContentType]:
    child_models = get_polymorphic_child_models(model)
    content_types = ContentType.objects.get_for_models(*child_models, for_concrete_models=True)
    return content_types


class PolymorphicCrudViewMixin:
    """
    Polymorphic ViewSet mixin
    """

    # todo: rename to cv_polymorphic_forms
    polymorphic_forms: Dict[Model, ModelForm] = None

    # polymorphic_inline_formsets: Dict[Model, Dict[str, Any]] = None

    # todo: rename to cv_~
    @property
    def polymorphic_ctype_id(self) -> int:
        """
        Get the polymorphic content type id from the url kwargs
        """
        # is there a polymorphic_ctype_id from the url in the create view?
        polymorphic_ctype_id = self.kwargs.get("polymorphic_ctype_id")
        if not polymorphic_ctype_id:
            # no, get it from the object
            instance = self.get_object()  # from SingleObjectMixin
            polymorphic_ctype_id = instance.polymorphic_ctype_id
        return int(polymorphic_ctype_id)

    # todo: rename to cv_~
    @cached_property
    def polymorphic_model(self) -> Model:
        """
        Get the polymorphic model from polymorphic_ctype_id
        """
        polymorphic_ctype_id = self.polymorphic_ctype_id
        content_type = ContentType.objects.get(id=polymorphic_ctype_id)
        model = content_type.model_class()
        return model

    @property
    def model(self) -> Model:
        """
        Override the model property from the view mixin
        """
        return self.polymorphic_model

    def get_form_class(self):
        """
        Get the form class depending on the polymorphic model
        Note: this implies that object has already been set
        """

        model = self.polymorphic_model
        form = self.polymorphic_forms.get(model, None)
        if not form:
            raise ValueError(f"No form found for polymorphic model {self.object.__class__}")
        # todo: assert self.object is set
        return form

    # def get_polymorphic_inline_formsets(self):
    #     return getattr(self, "polymorphic_inline_formsets", None)
    #
    # def get_inline_formsets(self) -> Optional[Dict[str, Any]]:
    #     """
    #     Get the inline formsets with polymorphic model.
    #     Therefore, we to override the polymorphic model property.
    #     """
    #
    #     # no formsets defined?
    #     inline_formsets = self.get_polymorphic_inline_formsets()
    #     if inline_formsets is None:
    #         return None
    #
    #     if not isinstance(inline_formsets, dict):
    #         raise ValueError(f"Formsets for {self.__class__.__name__} must be a dict")
    #
    #     # get formsets for model
    #     formsets = inline_formsets.get(self.polymorphic_model)
    #     if not formsets:
    #         return None
    #
    #     # check config
    #     if not isinstance(formsets, dict):
    #         raise ValueError(f"Formsets for {self.__class__.__name__}.{self.polymorphic_model} must be a dict")
    #     for key, value in formsets.items():
    #         if not isinstance(key, str):
    #             raise ValueError(f"Formset key {key} "
    #                              f"for {self.__class__.__name__}.{self.polymorphic_model} "
    #                              f"must be a string")
    #         if not issubclass(value, forms.BaseFormSet):
    #             raise ValueError(f"Formset {key} "
    #                              f"for {self.__class__.__name__}.{self.polymorphic_model} "
    #                              f"must be a Formset")
    #
    #     # everything is fine
    #     return formsets



# Generated by Django 5.1.4 on 2025-01-20 07:24

import django.db.models.deletion
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0002_poly_polyone_polytwo'),
    ]

    operations = [
        migrations.AlterField(
            model_name='author',
            name='first_name',
            field=models.CharField(max_length=100, verbose_name='Vorname'),
        ),
        migrations.CreateModel(
            name='Detail',
            fields=[
                ('order', models.PositiveIntegerField(db_index=True, editable=False, verbose_name='order')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('integer', models.IntegerField(verbose_name='An integer field')),
                ('number', models.FloatField(verbose_name='A float field')),
                ('char', models.CharField(max_length=100, verbose_name='Text field')),
                ('text', models.TextField(verbose_name='Multiline Text')),
                ('boolean', models.BooleanField(default=None, null=True, verbose_name='A boolean value')),
                ('boolean_two', models.BooleanField(default=None, null=True, verbose_name='Another boolean value')),
                ('date', models.DateField(verbose_name='A date field')),
                ('date_time', models.DateTimeField(verbose_name='A date field with time')),
                ('created_dt', models.DateTimeField(auto_now_add=True)),
                ('modified_dt', models.DateTimeField(auto_now=True)),
                ('author', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='app.author', verbose_name='A foreign key field')),
                ('foo', models.ManyToManyField(to='app.foo', verbose_name='Foo selected')),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
        ),
    ]



from django.apps import AppConfig


class CrudViewsBootstrap5Config(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'crud_views_bootstrap5'

    def ready(self):
        pass
        # todo: run checks
        # import viewset.checks  # noqa



from crud_views.lib.view import CrudViewPermissionRequiredMixin

from ..views.delete import DeleteView


class PolymorphicDeleteView(DeleteView):
    pass


class PolymorphicDeleteViewPermissionRequired(CrudViewPermissionRequiredMixin, PolymorphicDeleteView):  # this file
    cv_permission = "delete"



from .table import Column1, Column2, Column3, Column4, Column5, Column6, Column7, Column8, Column9, Column10, Column11, \
    Column12
from .form import CrispyModelForm, CrispyViewMixin, CrispyModelViewMixin, CrispyForm, CrispyDeleteForm

__all__ = [
    "Column1",
    "Column2",
    "Column3",
    "Column4",
    "Column5",
    "Column6",
    "Column7",
    "Column8",
    "Column9",
    "Column10",
    "Column11",
    "Column12",
    "CrispyModelForm",
    "CrispyForm",
    "CrispyDeleteForm",
    "CrispyModelViewMixin",
    "CrispyViewMixin",
]



from .table import Table, TableWithViewContext
from .columns import ActionColumn, LinkChildColumn, LinkDetailColumnMixin, LinkDetailColumn, UUIDColumn, \
    UUIDLinkDetailColumn

__all__ = [
    "Table",
    "TableWithViewContext",
    "ActionColumn",
    "LinkChildColumn",
    "LinkDetailColumnMixin",
    "LinkDetailColumn",
    "UUIDColumn",
    "UUIDLinkDetailColumn"
]



# Generated by Django 5.1.4 on 2024-12-27 13:45

import django.db.models.deletion
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Author',
            fields=[
                ('order', models.PositiveIntegerField(db_index=True, editable=False, verbose_name='order')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('first_name', models.CharField(max_length=100)),
                ('last_name', models.CharField(max_length=100)),
                ('pseudonym', models.CharField(blank=True, max_length=100, null=True)),
                ('created_dt', models.DateTimeField(auto_now_add=True)),
                ('modified_dt', models.DateTimeField(auto_now=True)),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Bar',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Foo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='Baz',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('bar', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app.bar')),
            ],
        ),
        migrations.CreateModel(
            name='Book',
            fields=[
                ('order', models.PositiveIntegerField(db_index=True, editable=False, verbose_name='order')),
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=100)),
                ('price', models.DecimalField(decimal_places=2, max_digits=10)),
                ('created_dt', models.DateTimeField(auto_now_add=True)),
                ('modified_dt', models.DateTimeField(auto_now=True)),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app.author')),
            ],
            options={
                'ordering': ('order',),
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='bar',
            name='foo',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='app.foo'),
        ),
    ]



from crud_views.lib.view import CrudViewPermissionRequiredMixin
from .utils import PolymorphicCrudViewMixin
from ..views import UpdateView


class PolymorphicUpdateView(PolymorphicCrudViewMixin, UpdateView):
    pass


class PolymorphicUpdateViewPermissionRequired(CrudViewPermissionRequiredMixin, PolymorphicUpdateView):  # this file
    cv_permission = "change"



import django_tables2 as tables
from django.forms import modelform_factory

from app.models import Poly, PolyOne, PolyTwo
from crud_views.lib.polymorphic_views import PolymorphicCreateSelectView, PolymorphicCreateViewPermissionRequired, \
    PolymorphicUpdateViewPermissionRequired, PolymorphicDetailViewPermissionRequired
from crud_views.lib.views import ListViewPermissionRequired
from crud_views.lib.polymorphic_views.delete import PolymorphicDeleteViewPermissionRequired
from crud_views.lib.table import Table
from crud_views.lib.views import ListViewTableMixin
from crud_views.lib.viewset import ViewSet, path_regs

cv_poly = ViewSet(
    model=Poly,
    name="poly",
    pk=path_regs.UUID,
)


class PolyTable(Table):
    id = tables.Column()
    shared = tables.Column()


class PolyListView(ListViewTableMixin, ListViewPermissionRequired):
    model = Poly
    table_class = PolyTable
    cv_viewset = cv_poly
    cv_list_actions = [
        "detail",
        "update",
        "delete"
    ]
    cv_context_actions = ["create_select", ]


class PolyCreateSelectView(PolymorphicCreateSelectView):
    model = Poly
    # fields = ["shared"]
    cv_viewset = cv_poly


PolyOneFormCreate = modelform_factory(
    PolyOne,
    fields=[
        "shared",
        "one",
    ],
)

PolyTwoFormCreate = modelform_factory(
    PolyTwo,
    fields=[
        "shared",
        "two",
    ],
)


class PolyCreateView(PolymorphicCreateViewPermissionRequired):
    model = Poly
    cv_viewset = cv_poly
    polymorphic_forms = {
        PolyOne: PolyOneFormCreate,
        PolyTwo: PolyTwoFormCreate
    }


PolyOneFormUpdate = modelform_factory(
    PolyOne,
    fields=[
        "shared",
        "one",
    ],
)

PolyTwoFormUpdate = modelform_factory(
    PolyTwo,
    fields=[
        "shared",
        "two",
    ],
)


class PolyUpdateView(PolymorphicUpdateViewPermissionRequired):
    model = Poly
    cv_viewset = cv_poly
    polymorphic_forms = {
        PolyOne: PolyOneFormUpdate,
        PolyTwo: PolyTwoFormUpdate
    }


class PolyDeleteView(PolymorphicDeleteViewPermissionRequired):
    model = Poly
    cv_viewset = cv_poly


class PolyDetailView(PolymorphicDetailViewPermissionRequired):
    model = Poly
    cv_viewset = cv_poly
    property_display = [
        {
            "title": "Properties",
            "properties": ["id", "shared"],
        },
    ]




import uuid

from django.db import models
from ordered_model.models import OrderedModel
from polymorphic.models import PolymorphicModel


class Author(OrderedModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    first_name = models.CharField(max_length=100)
    last_name = models.CharField(max_length=100)
    pseudonym = models.CharField(max_length=100, blank=True, null=True)

    def __str__(self):
        return f"{self.first_name} {self.last_name}"

    @property
    def full_name(self):
        return f"{self.first_name} {self.last_name}"


class Publisher(models.Model):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name


class Book(models.Model):
    title = models.CharField(max_length=200)
    publisher = models.ForeignKey(Publisher, on_delete=models.CASCADE, related_name="books")

    def __str__(self):
        return self.title


class Vehicle(PolymorphicModel):
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name


class Car(Vehicle):
    doors = models.IntegerField(default=4)


class Truck(Vehicle):
    payload_tons = models.IntegerField(default=10)



import django
import pytest
from django.conf import settings
from pathlib import Path

from django.test import Client

from tests.lib.helper.user import user_viewset_permission


def pytest_configure():
    settings.configure(

        BASE_DIR=Path(__file__).resolve().parent.parent,
        SECRET_KEY='django-testing',
        DEBUG=True,
        ALLOWED_HOSTS=[],
        INSTALLED_APPS=[
            'django.contrib.admin',
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.messages',
            'django.contrib.staticfiles',
            'django_bootstrap5',
            'crispy_forms',
            'crispy_bootstrap5',
            'polymorphic',
            'ordered_model',
            'django_tables2',
            "django_object_detail",
            "crud_views_bootstrap5.apps.CrudViewsBootstrap5Config",
            "crud_views.apps.CrudViewsConfig",
            'tests.test1.app',
        ],

        MIDDLEWARE=[
            'django.middleware.security.SecurityMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.common.CommonMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'django.middleware.clickjacking.XFrameOptionsMiddleware',
        ],

        ROOT_URLCONF='tests.test1.project.urls',

        TEMPLATES=[
            {
                'BACKEND': 'django.template.backends.django.DjangoTemplates',
                'DIRS': [],
                'APP_DIRS': True,
                'OPTIONS': {
                    'context_processors': [
                        'django.template.context_processors.debug',
                        'django.template.context_processors.request',
                        'django.contrib.auth.context_processors.auth',
                        'django.contrib.messages.context_processors.messages',
                        'crud_views.lib.context_processor.crud_views_context'
                    ],
                },
            },
        ],

        DATABASES={
            "default": {
                "ENGINE": "django.db.backends.sqlite3",
                "NAME": ":memory:",
            }
        },

        # Internationalization
        LANGUAGE_CODE='en-us',
        TIME_ZONE='UTC',
        USE_I18N=True,
        USE_TZ=True,

        STATIC_URL='static/',

        DEFAULT_AUTO_FIELD='django.db.models.BigAutoField',

        # Django ViewSet configuration
        CRUD_VIEWS_THEME="bootstrap5",
        CRUD_VIEWS_EXTENDS="app/crud_views.html",
        CRUD_VIEWS_MANAGE_VIEWS_ENABLED="yes",

        # crispy
        CRISPY_TEMPLATE_PACK="bootstrap5",
        CRISPY_ALLOWED_TEMPLATE_PACKS="bootstrap5",

        # django_tables2
        DJANGO_TABLES2_TEMPLATE="django_tables2/bootstrap5.html",
    )

    django.setup()


@pytest.fixture
def user_a():
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_a", password="password")
    return user


@pytest.fixture
def cv_author():
    from tests.test1.app.views import cv_author as ret
    return ret



@pytest.fixture
def user_author_view(cv_author):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_author_view", password="password")

    user_viewset_permission(user, cv_author, "view")

    return user


@pytest.fixture
def client() -> Client:
    return Client()


@pytest.fixture
def client_user_author_view(client, user_author_view) -> Client:
    client.force_login(user_author_view)
    return client


@pytest.fixture
def user_author_change(cv_author):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_author_change", password="password")

    user_viewset_permission(user, cv_author, "change")

    return user


@pytest.fixture
def user_author_delete(cv_author):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_author_delete", password="password")

    user_viewset_permission(user, cv_author, "delete")

    return user


@pytest.fixture
def author_douglas_adams():
    from tests.test1.app.models import Author

    return Author.objects.create(first_name="Douglas", last_name="Adams")


@pytest.fixture
def user_author_add(cv_author):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_author_add", password="password")

    user_viewset_permission(user, cv_author, "add")

    return user


@pytest.fixture
def client_user_author_change(client, user_author_change) -> Client:
    client.force_login(user_author_change)
    return client


@pytest.fixture
def client_user_author_delete(client, user_author_delete) -> Client:
    client.force_login(user_author_delete)
    return client


@pytest.fixture
def client_user_author_add(client, user_author_add) -> Client:
    client.force_login(user_author_add)
    return client


@pytest.fixture
def client_user_a(client, user_a) -> Client:
    client.force_login(user_a)
    return client


@pytest.fixture
def author_terry_pratchett():
    from tests.test1.app.models import Author

    return Author.objects.create(first_name="Terry", last_name="Pratchett")


# --- Publisher fixtures ---

@pytest.fixture
def cv_publisher():
    from tests.test1.app.views import cv_publisher as ret
    return ret


@pytest.fixture
def publisher_penguin():
    from tests.test1.app.models import Publisher
    return Publisher.objects.create(name="Penguin")


@pytest.fixture
def publisher_harpercollins():
    from tests.test1.app.models import Publisher
    return Publisher.objects.create(name="HarperCollins")


@pytest.fixture
def user_publisher_view(cv_publisher):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_publisher_view", password="password")
    user_viewset_permission(user, cv_publisher, "view")
    return user


@pytest.fixture
def client_user_publisher_view(client, user_publisher_view) -> Client:
    client.force_login(user_publisher_view)
    return client


@pytest.fixture
def user_publisher_add(cv_publisher):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_publisher_add", password="password")
    user_viewset_permission(user, cv_publisher, "add")
    return user


@pytest.fixture
def client_user_publisher_add(client, user_publisher_add) -> Client:
    client.force_login(user_publisher_add)
    return client


@pytest.fixture
def user_publisher_change(cv_publisher):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_publisher_change", password="password")
    user_viewset_permission(user, cv_publisher, "change")
    return user


@pytest.fixture
def client_user_publisher_change(client, user_publisher_change) -> Client:
    client.force_login(user_publisher_change)
    return client


@pytest.fixture
def user_publisher_delete(cv_publisher):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_publisher_delete", password="password")
    user_viewset_permission(user, cv_publisher, "delete")
    return user


@pytest.fixture
def client_user_publisher_delete(client, user_publisher_delete) -> Client:
    client.force_login(user_publisher_delete)
    return client


# --- Book fixtures ---

@pytest.fixture
def cv_book():
    from tests.test1.app.views import cv_book as ret
    return ret


@pytest.fixture
def book_hitchhiker(publisher_penguin):
    from tests.test1.app.models import Book
    return Book.objects.create(title="The Hitchhiker's Guide to the Galaxy", publisher=publisher_penguin)


@pytest.fixture
def book_other_publisher(publisher_harpercollins):
    from tests.test1.app.models import Book
    return Book.objects.create(title="Other Book", publisher=publisher_harpercollins)


@pytest.fixture
def user_book_view(cv_book):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_book_view", password="password")
    user_viewset_permission(user, cv_book, "view")
    return user


@pytest.fixture
def client_user_book_view(client, user_book_view) -> Client:
    client.force_login(user_book_view)
    return client


@pytest.fixture
def user_book_add(cv_book):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_book_add", password="password")
    user_viewset_permission(user, cv_book, "add")
    return user


@pytest.fixture
def client_user_book_add(client, user_book_add) -> Client:
    client.force_login(user_book_add)
    return client


@pytest.fixture
def user_book_change(cv_book):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_book_change", password="password")
    user_viewset_permission(user, cv_book, "change")
    return user


@pytest.fixture
def client_user_book_change(client, user_book_change) -> Client:
    client.force_login(user_book_change)
    return client


@pytest.fixture
def user_book_delete(cv_book):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_book_delete", password="password")
    user_viewset_permission(user, cv_book, "delete")
    return user


@pytest.fixture
def client_user_book_delete(client, user_book_delete) -> Client:
    client.force_login(user_book_delete)
    return client


# --- Vehicle (polymorphic) fixtures ---

@pytest.fixture
def cv_vehicle():
    from tests.test1.app.views import cv_vehicle as ret
    return ret


@pytest.fixture
def car_sedan():
    from tests.test1.app.models import Car
    return Car.objects.create(name="Sedan", doors=4)


@pytest.fixture
def truck_semi():
    from tests.test1.app.models import Truck
    return Truck.objects.create(name="Semi", payload_tons=20)


@pytest.fixture
def user_vehicle_view(cv_vehicle):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_vehicle_view", password="password")
    user_viewset_permission(user, cv_vehicle, "view")
    return user


@pytest.fixture
def client_user_vehicle_view(client, user_vehicle_view) -> Client:
    client.force_login(user_vehicle_view)
    return client


@pytest.fixture
def user_vehicle_add(cv_vehicle):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_vehicle_add", password="password")
    user_viewset_permission(user, cv_vehicle, "add")
    return user


@pytest.fixture
def client_user_vehicle_add(client, user_vehicle_add) -> Client:
    client.force_login(user_vehicle_add)
    return client


@pytest.fixture
def user_vehicle_change(cv_vehicle):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_vehicle_change", password="password")
    user_viewset_permission(user, cv_vehicle, "change")
    return user


@pytest.fixture
def client_user_vehicle_change(client, user_vehicle_change) -> Client:
    client.force_login(user_vehicle_change)
    return client


@pytest.fixture
def user_vehicle_delete(cv_vehicle):
    from django.contrib.auth.models import User
    user = User.objects.create_user(username="user_vehicle_delete", password="password")
    user_viewset_permission(user, cv_vehicle, "delete")
    return user


@pytest.fixture
def client_user_vehicle_delete(client, user_vehicle_delete) -> Client:
    client.force_login(user_vehicle_delete)
    return client



# Generated by Django 5.1.4 on 2024-12-27 14:59

import django.db.models.deletion
import uuid
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('app', '0001_initial'),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Poly',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('shared', models.CharField(max_length=100)),
                ('polymorphic_ctype', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='polymorphic_%(app_label)s.%(class)s_set+', to='contenttypes.contenttype')),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
        ),
        migrations.CreateModel(
            name='PolyOne',
            fields=[
                ('poly_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='app.poly')),
                ('one', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('app.poly',),
        ),
        migrations.CreateModel(
            name='PolyTwo',
            fields=[
                ('poly_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='app.poly')),
                ('two', models.CharField(max_length=100)),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('app.poly',),
        ),
    ]
